import processing.serial.*;
import java.util.concurrent.*;

XBee xbee;
Queue<XBeeResponse> queue = new ConcurrentLinkedQueue<XBeeResponse>();
boolean message;
XBeeResponse response;

String senderAddress = "00:00:00:00:00:00:00:00";
String rxStr = "********";
String txStatus = "";
int rssi = 0;

void setup()
{
  size(640,150);
  colorMode(RGB,256);
  background(0,0,0);
  
  try {
    //optional.  set up logging
    PropertyConfigurator.configure(dataPath("")+"log4j.properties");
    xbee = new XBee();
    // replace with your COM port
    xbee.open("COM5", 115200);
    xbee.addPacketListener(new PacketListener() {
      public void processResponse(XBeeResponse response) {
        queue.offer(response);
      }
    }
    );
  } 
  catch (Exception e) {
    System.out.println("XBee failed to initialize");
    e.printStackTrace();
    System.exit(1);
  }
}

void draw(){
  // パケット読み取り
  try {
  readPackets();
  }
  catch (Exception e) {
    e.printStackTrace();
  }
  
  background(255);
  textAlign(LEFT);
  textSize(20);
  
  fill(0);
  text("RSSI", 10, 25);
  fill(0,0,255);
  text(String.format("%d", rssi) + "dB", 100, 25);
  
  fill(0);
  text("Address", 10, 50);
  fill(0,0,255);
  text(senderAddress, 100, 50);
  
  fill(0);
  text("Rx Data", 10, 75);
  fill(0,0,255);
  text(rxStr, 100, 75);
  
  fill(255,0,0);
  text(txStatus, 10, 100);
}

void readPackets() throws Exception {

  while ((response = queue.poll()) != null) {
    // we got something!
    try {
      if (response.getApiId() == ApiId.ZNET_RX_RESPONSE && !response.isError()) {
        ZNetRxResponse rxResponse = (ZNetRxResponse) response;
        
        // アドレスをintで取得
        int[] addressArray = rxResponse.getRemoteAddress64().getAddress();
        String[] hexAddress = new String[addressArray.length];
        // 16進数のStringへ
        for(int i=0; i<addressArray.length;i++){
          hexAddress[i] = String.format("%02x",addressArray[i]);
        }
        // 送信元アドレスを文字列で保存
        senderAddress = join(hexAddress,":");
        
        //受信データを取得
        int[] rxData = rxResponse.getData();
        rxStr = "";
        for(int i=0; i<rxData.length; i++){
          rxStr += String.format("%c", rxData[i]);
        }
        
        // エコー
        int[] payload = rxData;
        XBeeAddress64 addr64 = new XBeeAddress64(addressArray);
        ZNetTxRequest request = new ZNetTxRequest(addr64, payload);
        try {
          ZNetTxStatusResponse response = (ZNetTxStatusResponse) xbee.sendSynchronous(request);
          txStatus = "";
        } catch (Exception e) {
          txStatus = "tx request timed out";
        }
        
        // ATコマンド送信
        // RSSI取得
        AtCommand  at = new AtCommand("DB");
        AtCommandResponse response = (AtCommandResponse) xbee.sendSynchronous(at, 5000);
        if (response.isOk()) {
          // 成功
          rssi = response.getValue()[0];
          rssi = rssi * (-1);
        }
      }
    }
    catch (ClassCastException e) {
      // not an IO Sample
    }
  }
}

